/*****************************************************************************
 * Filename:    options.h                                                    *
 * Description: Declaration of configuration structures                      *
 * Copyright:   2005 by Alexander Motzkau                                    *
 *****************************************************************************/

#ifndef __OPTIONS_H__
#define __OPTIONS_H__

struct storage_entry
{
    char *name;
    char *cipher;
    unsigned long size; /* 0 -> autodetect */
    unsigned long start;
    
    struct storage_entry *next;
};

struct storage
{
    char *device;
    struct storage_entry *entries;
    
    struct storage *next;
};

struct storage_block
{
    struct storage* device;
    struct storage_entry* entry;

    struct storage_block* next;
};

enum on_state
{
    on_success, on_failure, always
};

enum bool
{
    false = 0, true = 1
};

enum keytype
{
    ref, composite, passphrase, file, program, literal
};

enum nlmode
{
    nl_unchanged, nl_add, nl_remove
};

enum ttytype
{
    current, fixed, vt, xterm
};

struct command
{
    enum on_state on_state;
    enum bool ignore_status;
    enum bool needtty;
    char *uid;
    
    char *cmd;
    
    struct command *next;
};

struct key
{
    char *id;
    enum keytype type;
    unsigned int len; /* 0 -> default (256 bits) */
    
    struct key *next;
    struct key *subkeys;
    
    char *algo;
    char *value;

    enum bool ignore_status;
    enum bool needtty;
    enum nlmode newline;
    char *uid;

    void *result;
    int resultlen; /* key length in bytes */
};

struct map
{
    char *name;

    enum bool fsck;
    enum bool mount;

    char *point;
    char *fs;
    char *options;
    
    struct command *preexec;
    struct command *postexec;
    
    struct key *key;
    
    enum bool done; /* set by action.c */
    
    struct map *next;
};

struct action
{
    char *name;
    
    struct command *preexec;
    struct command *postexec;
    
    struct key *key;
    
    struct map *mappings;
    
    struct action *next;
};

struct tty
{
    enum ttytype type;
        
    struct command *preexec;
    struct command *postexec;
    
    char *dev;
    char *cmd;
    char *display;
    char *authority;
    
    int min;
    
    struct tty *next;
};

struct configuration
{
    struct tty *ttys;
    struct action *actions;
    struct storage *storage;
    
    void *action_tree;
    void *storage_tree;
    void *key_tree;
};

enum bool complete_config(struct configuration* config);

struct action* find_action(char *name, struct configuration* config);
struct storage_block* find_storage(char *name, struct configuration* config);
struct key* find_key(char *name, struct configuration* config);

void free_entry(struct storage_entry* entry);
void free_storage(struct storage* storage);
void free_key(struct key *key);
void free_command(struct command *cmd);
void free_map(struct map *map);
void free_action(struct action* act);
void free_tty(struct tty* tty);
void free_config(struct configuration *config);
#endif
